class DataObject<T> {
    // Overview: coppia <nomeDato, Dato>
    // IR: name, info =! null
    
    private String name;
    private T info;
    
    public DataObject (String nome, T inf) {
        if ((nome == null) || (inf == null))
            throw new NullPointerException();
        
        this.name = nome; this.info = inf;
    }
    
    /* restituisce il nome del dato */
    public String getName() { return name; }
    
    /* restituisce il dato */
    public T getInfo() { return info; }
    
    /* restituisce true se this coincide con e */
    public boolean equals(DataObject<T> dObj) throws NullPointerException {
        if (dObj == null) throw new NullPointerException();
        
        return this.name.equals(dObj.getName()) && this.info.equals(dObj.getInfo());
    };
}

public interface Container <T> {
    // Overview: Tipo modificabile che rappresenta un insieme modificabile
    //           <dato, n. versionr>
    // Typical Element: { <d1,v1>, ..., <dn, vn> } insieme di coppie con
    //                  (forall 0 <= j < n. (dj != null && vj >=0)) &&
    //                  (forall 0 <= i, j < n. di.nome = dj.nome => vi =! vj)

    /* inserisce una versione di dObj in this */
    public void put(DataObject<T> dObj) throws NullPointerException;
    /*
     REQUIRES: dObj != null
     THROWS: se dObj = null lancia NullPointerException
     MODIFIES: this
     EFFECTS: inserisce dObj associando nv+1 dove nv e' l'intero piu' alto
              fra i dati <do,nv> presenti per i quali dObj.getName() = do.getName
     */

    /* restituisce il dataObject con nome numero versione, null altrimenti */
    public DataObject<T> get(String name, int version)
           throws NullPointerException, NegativeVersionException;
    /*
     REQUIRES: name != null && version >= 0
     THROWS: se name = null lancia NullPointerException
             se version < 0 lancia NegativeVersionException (non presente in Java, checked)
     EFFECTS: restituisce dObj se dObj.getName = name && <dObj,versione> e' presente,
              null altrimenti
     */
}

class NegativeVersionException extends Exception {
    public NegativeVersionException() {
        super();
    }
    
    public NegativeVersionException(String s) {
        super(s);
    }
}
